__version__ = '0.1.6'
__title__ = "Waf Web Finder"
__description__ = "Automated web server (behind WAF) finder."
__url__ = "https://github.com/helviojunior/webfinder"
__build__ = 0x63c974a8
__author__ = "Helvio Junior (M4v3r1ck)"
__author_email__ = "helvio_junior@hotmail.com"
__license__ = "GPL-3.0"
__copyright__ = "Copyright Helvio Junior"
